@@ -2,8 +2,8 @@
 
    This file is part of the UPX executable compressor.
 
-   Copyright (C) 1996-2017 Markus Franz Xaver Johannes Oberhumer
-   Copyright (C) 1996-2017 Laszlo Molnar
+   Copyright (C) 1996-2018 Markus Franz Xaver Johannes Oberhumer
+   Copyright (C) 1996-2018 Laszlo Molnar
    All Rights Reserved.
 
    UPX and the UCL library are free software; you can redistribute them
@@ -592,38 +592,10 @@ void ElfLinkerAMD64::relocate1(const Relocation *rel, upx_byte *location, upx_ui
         super::relocate1(rel, location, value, type);
 }
 
-void ElfLinkerARM64::relocate1(const Relocation *rel, upx_byte *location, upx_uint64_t value,
-                               const char *type) {
-    if (strncmp(type, "R_AARCH64_", 10))
-        return super::relocate1(rel, location, value, type);
-    type += 10;
-
-    if (!strncmp(type, "PREL", 4)) {
-        value -= rel->section->offset + rel->offset;
-        type += 4;
-
-        if (!strcmp(type, "16"))
-            set_le16(location, get_le16(location) + value);
-        else if (!strncmp(type, "32", 2)) // for "32" and "32S"
-            set_le32(location, get_le32(location) + value);
-        else if (!strcmp(type, "64"))
-            set_le64(location, get_le64(location) + value);
-    } else if (!strcmp(type, "ABS32")) {
-        set_le32(location, get_le32(location) + value);
-    } else if (!strcmp(type, "ABS64")) {
-        set_le64(location, get_le64(location) + value);
-    } else if (!strcmp(type, "CONDBR19")) {
-        value -= rel->section->offset + rel->offset;
-        upx_uint32_t const m19 = ~(~0u << 19);
-        upx_uint32_t w = get_le32(location);
-        set_le32(location, (w & ~(m19 << 5)) | ((((w >> 5) + (value >> 2)) & m19) << 5));
-    } else
-        super::relocate1(rel, location, value, type);
-}
-
 void ElfLinkerArmBE::relocate1(const Relocation *rel, upx_byte *location, upx_uint64_t value,
                                const char *type) {
-    if (strcmp(type, "R_ARM_PC24") == 0) {
+    if (!strcmp(type, "R_ARM_PC24") || !strcmp(type, "R_ARM_CALL") ||
+        !strcmp(type, "R_ARM_JUMP24")) {
         value -= rel->section->offset + rel->offset;
         set_be24(1 + location, get_be24(1 + location) + value / 4);
     } else if (strcmp(type, "R_ARM_ABS32") == 0) {
@@ -647,7 +619,8 @@ void ElfLinkerArmBE::relocate1(const Relocation *rel, upx_byte *location, upx_ui
 
 void ElfLinkerArmLE::relocate1(const Relocation *rel, upx_byte *location, upx_uint64_t value,
                                const char *type) {
-    if (strcmp(type, "R_ARM_PC24") == 0) {
+    if (!strcmp(type, "R_ARM_PC24") || !strcmp(type, "R_ARM_CALL") ||
+        !strcmp(type, "R_ARM_JUMP24")) {
         value -= rel->section->offset + rel->offset;
         set_le24(location, get_le24(location) + value / 4);
     } else if (strcmp(type, "R_ARM_ABS32") == 0) {
@@ -671,11 +644,40 @@ void ElfLinkerArmLE::relocate1(const Relocation *rel, upx_byte *location, upx_ui
 
 void ElfLinkerArm64LE::relocate1(const Relocation *rel, upx_byte *location, upx_uint64_t value,
                                  const char *type) {
-    if (strcmp(type, "R_AARCH64_CALL26") == 0) {
+    if (strncmp(type, "R_AARCH64_", 10))
+        return super::relocate1(rel, location, value, type);
+    type += 10;
+
+    if (!strncmp(type, "PREL", 4)) {
         value -= rel->section->offset + rel->offset;
-        set_le24(location, get_le24(location) + value / 4); // FIXME set_le26
-    } else if (strcmp(type, "R_AARCH64_ABS32") == 0) {
+        type += 4;
+
+        if (!strcmp(type, "16"))
+            set_le16(location, get_le16(location) + value);
+        else if (!strncmp(type, "32", 2)) // for "32" and "32S"
+            set_le32(location, get_le32(location) + value);
+        else if (!strcmp(type, "64"))
+            set_le64(location, get_le64(location) + value);
+    } else if (!strcmp(type, "ADR_PREL_LO21")) {
+        value -= rel->section->offset + rel->offset;
+        upx_uint32_t const m19 = ~(~0u << 19);
+        upx_uint32_t w = get_le32(location);
+        set_le32(location, (w & ~((3u << 29) | (m19 << 5))) | ((3u & value) << 29) |
+                               ((m19 & (value >> 2)) << 5));
+    } else if (!strcmp(type, "ABS32")) {
         set_le32(location, get_le32(location) + value);
+    } else if (!strcmp(type, "ABS64")) {
+        set_le64(location, get_le64(location) + value);
+    } else if (!strcmp(type, "CONDBR19")) {
+        value -= rel->section->offset + rel->offset;
+        upx_uint32_t const m19 = ~(~0u << 19);
+        upx_uint32_t w = get_le32(location);
+        set_le32(location, (w & ~(m19 << 5)) | ((((w >> 5) + (value >> 2)) & m19) << 5));
+    } else if (!strcmp(type, "CALL26")) {
+        value -= rel->section->offset + rel->offset;
+        upx_uint32_t const m26 = ~(~0u << 26);
+        upx_uint32_t w = get_le32(location);
+        set_le32(location, (w & ~m26) | (m26 & (value >> 2)));
     } else
         super::relocate1(rel, location, value, type);
 }
@@ -797,6 +799,14 @@ void ElfLinkerPpc32::relocate1(const Relocation *rel, upx_byte *location, upx_ui
 
 void ElfLinkerPpc64le::relocate1(const Relocation *rel, upx_byte *location, upx_uint64_t value,
                                  const char *type) {
+    if (!strcmp(type, "R_PPC64_ADDR64")) {
+        set_le64(location, get_le64(location) + value);
+        return;
+    }
+    if (!strcmp(type, "R_PPC64_ADDR32")) {
+        set_le32(location, get_le32(location) + value);
+        return;
+    }
     if (strncmp(type, "R_PPC64_REL", 11))
         return super::relocate1(rel, location, value, type);
     type += 11;
@@ -816,11 +826,11 @@ void ElfLinkerPpc64le::relocate1(const Relocation *rel, upx_byte *location, upx_
             internal_error("target out of range (%d) in reloc %s:%x\n", displ, rel->section->name,
                            rel->offset);
         *location += value;
-    } else if (strncmp(type, "14", 2) == 0) // for "32" and "32S"
+    } else if (strncmp(type, "14", 2) == 0) // for "14" and "14S"
         set_le16(location, get_le16(location) + value);
     else if (strcmp(type, "16") == 0)
         set_le16(location, get_le16(location) + value);
-    else if (strncmp(type, "24", 2) == 0) // for "32" and "32S"
+    else if (strncmp(type, "24", 2) == 0) // for "24" and "24S"
         set_le24(location, get_le24(location) + value);
     else if (strncmp(type, "32", 2) == 0) // for "32" and "32S"
         set_le32(location, get_le32(location) + value);
@@ -830,6 +840,36 @@ void ElfLinkerPpc64le::relocate1(const Relocation *rel, upx_byte *location, upx_
         super::relocate1(rel, location, value, type);
 }
 
+void ElfLinkerPpc64::relocate1(const Relocation *rel, upx_byte *location, upx_uint64_t value,
+                               const char *type) {
+    if (!strcmp(type, "R_PPC64_ADDR32")) {
+        set_be32(location, get_be32(location) + value);
+        return;
+    }
+    if (!strcmp(type, "R_PPC64_ADDR64")) {
+        set_be64(location, get_be64(location) + value);
+        return;
+    }
+    if (strncmp(type, "R_PPC64_REL", 11))
+        return super::relocate1(rel, location, value, type);
+    type += 11;
+
+    if (strncmp(type, "PC", 2) == 0) {
+        type += 2;
+    }
+
+    // We have "R_PPC64_REL" or "R_PPC64_RELPC" as a prefix.
+    /* value will hold relative displacement */
+    value -= rel->section->offset + rel->offset;
+
+    if (strncmp(type, "14", 2) == 0) // for "14" and "14S"
+        set_be16(2 + location, get_be16(2 + location) + value);
+    else if (strncmp(type, "24", 2) == 0) // for "24" and "24S"
+        set_be24(1 + location, get_be24(1 + location) + value);
+    else
+        super::relocate1(rel, location, value, type);
+}
+
 void ElfLinkerX86::relocate1(const Relocation *rel, upx_byte *location, upx_uint64_t value,
                              const char *type) {
     if (strncmp(type, "R_386_", 6))
