@@ -2,9 +2,9 @@
 
    This file is part of the UPX executable compressor.
 
-   Copyright (C) 1996-2017 Markus Franz Xaver Johannes Oberhumer
-   Copyright (C) 1996-2017 Laszlo Molnar
-   Copyright (C) 2000-2017 John F. Reiser
+   Copyright (C) 1996-2018 Markus Franz Xaver Johannes Oberhumer
+   Copyright (C) 1996-2018 Laszlo Molnar
+   Copyright (C) 2000-2018 John F. Reiser
    All Rights Reserved.
 
    UPX and the UCL library are free software; you can redistribute them
@@ -51,7 +51,7 @@ protected:
 
     virtual void pack1(OutputFile *, Filter &) = 0;  // generate executable header
     virtual int  pack2(OutputFile *, Filter &) = 0;  // append compressed data
-    virtual void pack3(OutputFile *, Filter &) = 0;  // append loader
+    virtual off_t pack3(OutputFile *, Filter &) = 0;  // append loader
     //virtual void pack4(OutputFile *, Filter &) = 0;  // append pack header
 
     virtual void generateElfHdr(
@@ -68,6 +68,7 @@ protected:
     static unsigned gnu_hash(char const *) /*const*/;
 
 protected:
+    static unsigned int const asl_delta = (1u<<12);  // --android-shlib extra page
     unsigned e_type;
     unsigned e_phnum;       /* Program header table entry count */
     unsigned e_shnum;
@@ -80,18 +81,24 @@ protected:
     unsigned sz_pack2a;  // after pack2() of all PT_LOAD
     unsigned lg2_page;  // log2(PAGE_SIZE)
     unsigned page_size;  // 1u<<lg2_page
-    bool is_big;  // stub style: must use area above the brk
+    bool is_pie;  // is Position-Independent-Executable (ET_DYN main program)
     unsigned xct_off;  // shared library: file offset of SHT_EXECINSTR
     unsigned hatch_off;  // file offset of escape hatch
+    unsigned o_binfo;  // offset to first b_info
     upx_uint64_t load_va;  // PT_LOAD[0].p_vaddr
     upx_uint64_t xct_va;  // minimum SHT_EXECINSTR virtual address
     upx_uint64_t jni_onload_va;  // runtime &JNI_OnLoad
+    upx_uint64_t user_init_va;
+    unsigned user_init_off;  // within file_image
 
     upx_uint16_t  e_machine;
     unsigned char ei_class;
     unsigned char ei_data;
     unsigned char ei_osabi;
     char const *osabi_note;
+    unsigned upx_dt_init;  // DT_INIT, DT_PREINIT_ARRAY, DT_INIT_ARRAY
+    static unsigned const DT_NUM = 34;  // elf.h
+    unsigned dt_table[DT_NUM];  // 1+ index in PT_DYNAMIC
 
     unsigned char const *buildid_data;
     int o_elf_shnum; // num output Shdrs
@@ -117,15 +124,18 @@ protected:
 
     virtual void pack1(OutputFile *, Filter &);  // generate executable header
     virtual int  pack2(OutputFile *, Filter &);  // append compressed data
-    virtual void pack3(OutputFile *, Filter &);  // append loader
+    virtual off_t pack3(OutputFile *, Filter &);  // append loader
     virtual void pack4(OutputFile *, Filter &);  // append pack header
     virtual void unpack(OutputFile *fo);
+    virtual void unRel32(unsigned dt_rel, Elf32_Rel *rel0, unsigned relsz,
+        MemBuffer &membuf, unsigned const load_off, OutputFile *fo);
 
     virtual void generateElfHdr(
         OutputFile *,
         void const *proto,
         unsigned const brka
     );
+    virtual void defineSymbols(Filter const *);
     virtual void buildLinuxLoader(
         upx_byte const *const proto,  // assembly-only sections
         unsigned const szproto,
@@ -139,26 +149,33 @@ protected:
     virtual unsigned find_LOAD_gap(Elf32_Phdr const *const phdri, unsigned const k,
         unsigned const e_phnum);
     virtual off_t getbase(const Elf32_Phdr *phdr, int e_phnum) const;
+    bool calls_crt1(Elf32_Rel const *rel, int sz);
 
     virtual Elf32_Sym const *elf_lookup(char const *) const;
     virtual unsigned elf_get_offset_from_address(unsigned) const;
     Elf32_Phdr const *elf_find_ptype(unsigned type, Elf32_Phdr const *phdr0, unsigned phnum);
     Elf32_Shdr const *elf_find_section_name(char const *) const;
     Elf32_Shdr const *elf_find_section_type(unsigned) const;
+    unsigned check_pt_dynamic(Elf32_Phdr const *);
+    void invert_pt_dynamic(Elf32_Dyn const *);
     void const *elf_find_dynamic(unsigned) const;
     Elf32_Dyn const *elf_has_dynamic(unsigned) const;
     virtual upx_uint64_t elf_unsigned_dynamic(unsigned) const;
 
 protected:
     Elf32_Ehdr  ehdri; // from input file
+    MemBuffer lowmem;  // especially for shlib
     Elf32_Phdr *phdri; // for  input file
-    Elf32_Shdr const *shdri; // from input file
+    Elf32_Shdr *shdri; // from input file
     Elf32_Phdr const *gnu_stack;  // propagate NX
     unsigned e_phoff;
     unsigned e_shoff;
+    unsigned sz_dynseg;  // PT_DYNAMIC.p_memsz
     unsigned so_slide;
     unsigned char *note_body;  // concatenated contents of PT_NOTEs, if any
     unsigned note_size;  // total size of PT_NOTEs
+    unsigned n_jmp_slot;
+    unsigned plt_off;
     unsigned page_mask;  // AND clears the offset-within-page
 
     Elf32_Dyn    const *dynseg;   // from PT_DYNAMIC
@@ -168,7 +185,7 @@ protected:
     Elf32_Sym    const *jni_onload_sym;
     char const *shstrtab;   // via Elf32_Shdr
 
-    Elf32_Shdr const *sec_strndx;
+    Elf32_Shdr       *sec_strndx;
     Elf32_Shdr const *sec_dynsym;
     Elf32_Shdr const *sec_dynstr;
 
@@ -235,18 +252,22 @@ public:
 protected:
     virtual void PackLinuxElf64help1(InputFile *f);
     virtual int checkEhdr(Elf64_Ehdr const *ehdr) const;
+    virtual bool canPack();
 
     virtual void pack1(OutputFile *, Filter &);  // generate executable header
     virtual int  pack2(OutputFile *, Filter &);  // append compressed data
-    virtual void pack3(OutputFile *, Filter &);  // append loader
+    virtual off_t pack3(OutputFile *, Filter &);  // append loader
     virtual void pack4(OutputFile *, Filter &);  // append pack header
     virtual void unpack(OutputFile *fo);
+    virtual void unRela64(upx_uint64_t dt_rela, Elf64_Rela *rela0, unsigned relasz,
+        MemBuffer &membuf, upx_uint64_t const load_off, OutputFile *fo);
 
     virtual void generateElfHdr(
         OutputFile *,
         void const *proto,
         unsigned const brka
     );
+    virtual void defineSymbols(Filter const *);
     virtual void buildLinuxLoader(
         upx_byte const *const proto,  // assembly-only sections
         unsigned const szproto,
@@ -259,26 +280,33 @@ protected:
     virtual void updateLoader(OutputFile *fo);
     virtual unsigned find_LOAD_gap(Elf64_Phdr const *const phdri, unsigned const k,
         unsigned const e_phnum);
+    bool calls_crt1(Elf64_Rela const *rela, int sz);
 
     virtual Elf64_Sym const *elf_lookup(char const *) const;
     virtual upx_uint64_t elf_get_offset_from_address(upx_uint64_t) const;
     Elf64_Phdr const *elf_find_ptype(unsigned type, Elf64_Phdr const *phdr0, unsigned phnum);
     Elf64_Shdr const *elf_find_section_name(char const *) const;
     Elf64_Shdr const *elf_find_section_type(unsigned) const;
+    upx_uint64_t check_pt_dynamic(Elf64_Phdr const *);
+    void invert_pt_dynamic(Elf64_Dyn const *);
     void const *elf_find_dynamic(unsigned) const;
     Elf64_Dyn const *elf_has_dynamic(unsigned) const;
     virtual upx_uint64_t elf_unsigned_dynamic(unsigned) const;
 
 protected:
     Elf64_Ehdr  ehdri; // from input file
+    MemBuffer lowmem;  // especially for shlib
     Elf64_Phdr *phdri; // for  input file
-    Elf64_Shdr const *shdri; // from input file
+    Elf64_Shdr *shdri; // from input file
     Elf64_Phdr const *gnu_stack;  // propagate NX
     upx_uint64_t e_phoff;
     upx_uint64_t e_shoff;
-    unsigned so_slide;
+    upx_uint64_t sz_dynseg;  // PT_DYNAMIC.p_memsz
+    upx_uint64_t so_slide;
     unsigned char *note_body;  // concatenated contents of PT_NOTEs, if any
     unsigned note_size;  // total size of PT_NOTEs
+    unsigned n_jmp_slot;
+    upx_uint64_t plt_off;
     upx_uint64_t page_mask;  // AND clears the offset-within-page
 
     Elf64_Dyn    const *dynseg;   // from PT_DYNAMIC
@@ -288,7 +316,7 @@ protected:
     Elf64_Sym    const *jni_onload_sym;
     char const *shstrtab;   // via Elf64_Shdr
 
-    Elf64_Shdr const *sec_strndx;
+    Elf64_Shdr       *sec_strndx;
     Elf64_Shdr const *sec_dynsym;
     Elf64_Shdr const *sec_dynstr;
 
@@ -368,6 +396,18 @@ protected:
     }
 };
 
+class PackLinuxElf64Be : public PackLinuxElf64
+{
+    typedef PackLinuxElf64 super;
+protected:
+    PackLinuxElf64Be(InputFile *f) : super(f) {
+        lg2_page=16;
+        page_size=1u<<lg2_page;
+        bele = &N_BELE_RTP::be_policy;
+        PackLinuxElf64help1(f);
+    }
+};
+
 
 /*************************************************************************
 // linux/elf64amd
@@ -383,10 +423,8 @@ public:
     virtual const char *getName() const { return "linux/amd64"; }
     virtual const char *getFullName(const options_t *) const { return "amd64-linux.elf"; }
     virtual const int *getFilters() const;
-    virtual bool canPack();
 protected:
     virtual void pack1(OutputFile *, Filter &);  // generate executable header
-    //virtual void pack3(OutputFile *, Filter &);  // append loader
     virtual void buildLoader(const Filter *);
     virtual Linker* newLinker() const;
     virtual void defineSymbols(Filter const *);
@@ -402,10 +440,8 @@ public:
     virtual const char *getName() const { return "linux/arm64"; }
     virtual const char *getFullName(const options_t *) const { return "arm64-linux.elf"; }
     virtual const int *getFilters() const;
-    virtual bool canPack();
 protected:
     virtual void pack1(OutputFile *, Filter &);  // generate executable header
-    //virtual void pack3(OutputFile *, Filter &);  // append loader
     virtual void buildLoader(const Filter *);
     virtual Linker* newLinker() const;
     virtual void defineSymbols(Filter const *);
@@ -449,7 +485,25 @@ public:
 protected:
     unsigned lg2_page;  // log2(PAGE_SIZE)
     unsigned page_size;  // 1u<<lg2_page
-    virtual bool canPack();
+    virtual void pack1(OutputFile *, Filter &);  // generate executable header
+    virtual void buildLoader(const Filter *);
+    virtual Linker* newLinker() const;
+};
+
+
+class PackLinuxElf64ppc : public PackLinuxElf64Be
+{
+    typedef PackLinuxElf64Be super;
+public:
+    PackLinuxElf64ppc(InputFile *f);
+    virtual ~PackLinuxElf64ppc();
+    virtual int getFormat() const { return UPX_F_LINUX_ELFPPC64; }
+    virtual const char *getName() const { return "linux/ppc64"; }
+    virtual const char *getFullName(const options_t *) const { return "powerpc64-linux.elf"; }
+    virtual const int *getFilters() const;
+protected:
+    unsigned lg2_page;  // log2(PAGE_SIZE)
+    unsigned page_size;  // 1u<<lg2_page
     virtual void pack1(OutputFile *, Filter &);  // generate executable header
     virtual void buildLoader(const Filter *);
     virtual Linker* newLinker() const;
