@@ -2,9 +2,9 @@
 
    This file is part of the UPX executable compressor.
 
-   Copyright (C) 1996-2017 Markus Franz Xaver Johannes Oberhumer
-   Copyright (C) 1996-2017 Laszlo Molnar
-   Copyright (C) 2000-2017 John F. Reiser
+   Copyright (C) 1996-2018 Markus Franz Xaver Johannes Oberhumer
+   Copyright (C) 1996-2018 Laszlo Molnar
+   Copyright (C) 2000-2018 John F. Reiser
    All Rights Reserved.
 
    UPX and the UCL library are free software; you can redistribute them
@@ -246,7 +246,7 @@ PackUnix::patchLoaderChecksum()
     set_te32(&lp->l_checksum, upx_adler32(ptr, lsize));
 }
 
-void PackUnix::pack3(OutputFile *fo, Filter &ft)
+off_t PackUnix::pack3(OutputFile *fo, Filter &ft)
 {
     if (0==linker) {
         // If no filter, then linker is not constructed by side effect
@@ -259,6 +259,7 @@ void PackUnix::pack3(OutputFile *fo, Filter &ft)
     updateLoader(fo);
     patchLoaderChecksum();
     fo->write(p, lsize);
+    return fo->getBytesWritten();
 }
 
 void PackUnix::pack4(OutputFile *fo, Filter &)
@@ -557,7 +558,10 @@ int PackUnix::canUnpack()
 
 void PackUnix::unpack(OutputFile *fo)
 {
-    unsigned const szb_info = sizeof(b_info);
+    b_info bhdr;
+    unsigned const szb_info = (ph.version <= 11)
+        ? sizeof(bhdr.sz_unc) + sizeof(bhdr.sz_cpr)  // old style
+        : sizeof(bhdr);
 
     unsigned c_adler = upx_adler32(NULL, 0);
     unsigned u_adler = upx_adler32(NULL, 0);
@@ -590,7 +594,7 @@ void PackUnix::unpack(OutputFile *fo)
     // decompress blocks
     unsigned total_in = 0;
     unsigned total_out = 0;
-    b_info bhdr; memset(&bhdr, 0, sizeof(bhdr));
+    memset(&bhdr, 0, sizeof(bhdr));
     for (;;)
     {
 #define buf ibuf
