@@ -2,9 +2,9 @@
 
    This file is part of the UPX executable compressor.
 
-   Copyright (C) 1996-2017 Markus Franz Xaver Johannes Oberhumer
-   Copyright (C) 1996-2017 Laszlo Molnar
-   Copyright (C) 2000-2017 John F. Reiser
+   Copyright (C) 1996-2018 Markus Franz Xaver Johannes Oberhumer
+   Copyright (C) 1996-2018 Laszlo Molnar
+   Copyright (C) 2000-2018 John F. Reiser
    All Rights Reserved.
 
    UPX and the UCL library are free software; you can redistribute them
@@ -346,6 +346,11 @@ auxv_up(Elf32_auxv_t *av, unsigned const type, unsigned const value)
             av->a_un.a_val = value;
             return;
         }
+        if (av->a_type==AT_NULL) {
+            // We can't do this as part of the for loop because we overwrite
+            // AT_NULL too.
+            return;
+        }
     }
 }
 
@@ -373,7 +378,7 @@ umax(unsigned a, unsigned b)
 // of lowering the segment limit on %cs as an implementation of "no-execute
 // .data".  Thus OpenBSD 3.9 puts a gap of 0x20000000 (512 MiB) between the
 // origins of .text and .data in each module.  So, mapping ET_DYN must "level"
-// the excursion in .text and .data of anything which preceeded it,
+// the excursion in .text and .data of anything which preceded it,
 // then add the new PT_LOAD.
 
 static unsigned long  // returns relocation constant
